{% extends 'admin/kmuhelper/change_form.html' %}

{% block object-tools-items %}
    {% if change %}
        {% if original.woocommerceid %}
            <li><a href="{% url 'kmuhelper:wc-update-order' object_id %}">Von WooCommerce importieren und ersetzen</a></li>
        {% endif %}
        <li><a href="{% url 'kmuhelper:bestellung-pdf-ansehen' object_id %}" target="_blank" rel="noopener">Rechnung (digital)</a></li>
        <li><a href="{% url 'kmuhelper:bestellung-pdf-ansehen' object_id %}?druck" target="_blank" rel="noopener">Rechnung (druck)</a></li>
        <li><a href="{% url 'kmuhelper:bestellung-pdf-ansehen' object_id %}?lieferschein" target="_blank" rel="noopener">Lieferschein</a></li>

        {% load kmuhelper_tags %}
        {% kmuhelper_email_show_buttons as show_email_buttons %}
        {% if show_email_buttons and original.versendet and original.rechnungsadresse_email %}
            {% if not original.rechnungsemail or not original.rechnungsemail.time_sent %}
                <li><a href="{% url 'kmuhelper:bestellung-pdf-an-kunden-senden' object_id %}">Rechnung an Kunden senden</a></li>
            {% endif %}
        {% endif %}
        
        {% if original.trackingnummer %}
            <li><a target="_blank" href="{{ original.trackinglink }}" rel="noopener">Tracking</a></li>
        {% endif %}

        {% if not original.versendet %}
            <li><a href="#id_lieferadresse_vorname" onclick="copyform();">Rechnungsadresse -> Lieferadresse</a></li>
            <script>
                function copyform(){
                    document.getElementById("id_lieferadresse_vorname").value = document.getElementById("id_rechnungsadresse_vorname").value;
                    document.getElementById("id_lieferadresse_nachname").value = document.getElementById("id_rechnungsadresse_nachname").value;
                    document.getElementById("id_lieferadresse_firma").value = document.getElementById("id_rechnungsadresse_firma").value;
                    document.getElementById("id_lieferadresse_adresszeile1").value = document.getElementById("id_rechnungsadresse_adresszeile1").value;
                    document.getElementById("id_lieferadresse_adresszeile2").value = document.getElementById("id_rechnungsadresse_adresszeile2").value;
                    document.getElementById("id_lieferadresse_ort").value = document.getElementById("id_rechnungsadresse_ort").value;
                    document.getElementById("id_lieferadresse_kanton").value = document.getElementById("id_rechnungsadresse_kanton").value;
                    document.getElementById("id_lieferadresse_plz").value = document.getElementById("id_rechnungsadresse_plz").value;
                    document.getElementById("id_lieferadresse_land").value = document.getElementById("id_rechnungsadresse_land").value;
                }
            </script>
        {% endif %}
    {% endif %}
    {{ block.super }}
{% endblock %}
