body {
    height: 100vh;
    width: 100vw;
    margin: 0;
    padding: 0;
}

#body {
    height: 90%;
    width: 100%;
    background-position: center center;
    background-repeat: no-repeat;
    background-size: contain;
}

iframe {
    height: 100%;
    width: 100%;
    border: none;
}

.appbar {
    width: 100%;
    height: 10%;
    display: flex;
    justify-content: space-evenly;
    align-items: center;
    overflow: hidden;
    position: static;
    background-color: #ffffff;
}

.appbar .item {
    text-decoration: none;
    color: #000000;
    font-size: 150%;
}

.appbar .item:target {
    color: #7c7c7c;
}

.appbar .item:hover {
    color: #dd1155;
}

.appbar .item.hidden {
    display: none;
}