{% extends 'kmuhelper/emails/base.html' %}

{% block title %}Ihre Rechnung #{{ id }}{% endblock %}

{% block header %}{% endblock %}

{% block content %}

    <p>Im Anhang finden Sie Ihre Rechnung zur Bestellung Nr. {{ id }}{% if woocommercedata %} (Online #{{ woocommerceid }}){% endif %}</p>

    {% if trackingdata %}
        <p>Sie können die Bestellung über <a href="{{ trackinglink }}">diesen Link</a> verfolgen, sobald Sie von der Post verarbeitet wurde.</p>
    {% endif %}

{% endblock %}