{% extends 'admin/kmuhelper/change_form.html' %}

{% block object-tools %}
    <ul class="object-tools">
        {% if change %}{% if not is_popup %}
            {% block object-tools-items %}
                {% if original.woocommerceid %}
                    <li><a href="{% url 'kmuhelper:wc-update-customer' object_id %}">Von WooCommerce importieren und ersetzen</a></li>
                {% endif %}

                {% load kmuhelper_tags %}
                {% kmuhelper_email_show_buttons as show_email_buttons %}
                {% if show_email_buttons and original.email and not original.registrierungsemail or not original.registrierungsemail.time_sent %}
                    <li><a href="{% url 'kmuhelper:kunde-email-registriert' object_id %}">Registrierungsemail senden</a></li>
                {% endif %}

                {{ block.super }}
            {% endblock %}
        {% endif %}{% endif %}

        <li><a href="#id_lieferadresse_vorname" onclick="copyform();">Rechnungsadresse -> Lieferadresse</a></li>
        <script>
        function copyform(){
            document.getElementById("id_lieferadresse_vorname").value = document.getElementById("id_rechnungsadresse_vorname").value;
            document.getElementById("id_lieferadresse_nachname").value = document.getElementById("id_rechnungsadresse_nachname").value;
            document.getElementById("id_lieferadresse_firma").value = document.getElementById("id_rechnungsadresse_firma").value;
            document.getElementById("id_lieferadresse_adresszeile1").value = document.getElementById("id_rechnungsadresse_adresszeile1").value;
            document.getElementById("id_lieferadresse_adresszeile2").value = document.getElementById("id_rechnungsadresse_adresszeile2").value;
            document.getElementById("id_lieferadresse_ort").value = document.getElementById("id_rechnungsadresse_ort").value;
            document.getElementById("id_lieferadresse_kanton").value = document.getElementById("id_rechnungsadresse_kanton").value;
            document.getElementById("id_lieferadresse_plz").value = document.getElementById("id_rechnungsadresse_plz").value;
            document.getElementById("id_lieferadresse_land").value = document.getElementById("id_rechnungsadresse_land").value;
        }
        </script>
    </ul>
{% endblock %}
