{% extends 'kmuhelper/emails/base.html' %}

{% block title %}Willkommen Herr/Frau <strong>{{ kunde.vorname }} {{ kunde.nachname }}</strong>!{% endblock %}

{% block header %}{% endblock %}

{% block content %}

    <p>Sie wurden erfolgreich in unserer Datenbank registriert!</p>

{% endblock %}